package com.aidlab.aidlab_sdk

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.aidlab.sdk.communication.*
import java.util.*


object AidlabBluetoothManager : AidlabSDKDelegate {

    private var initialized = false
    private var aidlabSDK: AidlabSDK? = null
    private var detectedAidlabs = emptyList<Aidlab>()
    var lastConnectedAidlab: Aidlab? = null
        private set
        
    var listener: PlatformAidlabSdkDelegate? = null

    fun init(context: Context) {
        if(initialized) return
        initialized = true

        Handler(Looper.getMainLooper()).post {
            aidlabSDK = AidlabSDK(context, this)
        }
    }

    fun startScan() {
        aidlabSDK?.scanForDevices()
    }

    fun stopScan() {
        aidlabSDK?.stopDeviceScan()
    }

    fun connect(address: String, signals: EnumSet<Signal>, aidlabDelegate: AidlabDelegate) {
        stopScan()
        val aidlab = addressToAidlab(address)
        lastConnectedAidlab = aidlab
        aidlab.connect(signals, false, aidlabDelegate)
    }

    fun addressToAidlab(address: String): Aidlab {
        return detectedAidlabs.find { it.address() == address }
                ?: throw IllegalStateException("Address not found in detected aidlabs($address)")
    }

    override fun onAidlabDetected(aidlab: Aidlab, rssi: Int) {
        detectedAidlabs = detectedAidlabs.filter { it.address() != aidlab.address() }
        detectedAidlabs = detectedAidlabs.plus(aidlab)

        listener?.onAidlabDetected(AidlabDetectedEvent(aidlab.address(), rssi.toLong())) {}
    }

    override fun onScanFailed(errorCode: Int) {
        listener?.onScanFailed(ScanFailedEvent(errorCode.toLong())) {}
    }

    override fun onBluetoothStarted() {
        listener?.onBluetoothStarted(EmptyEvent(0)) {}
    }
    override fun onDeviceScanStarted() {
        listener?.onDeviceScanStarted(EmptyEvent(0)) {}
    }
    override fun onDeviceScanStopped() {
        listener?.onDeviceScanStopped(EmptyEvent(0)) {}
    }
}