package com.aidlab.aidlab_sdk

import com.aidlab.sdk.communication.*
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger

class AidlabSynchronizationHandler(
        binaryMessenger: BinaryMessenger
): AidlabSynchronizationDelegate {
    
    private var listener: PlatformAidlabSynchronizationDelegate? = null
    
    init {
        listener = PlatformAidlabSynchronizationDelegate(binaryMessenger)
    }

    fun release() {
        listener = null
    }

    override fun didDetectPastUserEvent(timestamp: Long) {
        listener?.didDetectPastUserEvent(UserEvent(timestamp)) {}
    }

    override fun didReceivePastAccelerometer(aidlab: IAidlab, timestamp: Long, ax: Float, ay: Float, az: Float) {
        listener?.didReceivePastAccelerometer(VectorEvent(timestamp, ax.toDouble(), ay.toDouble(), az.toDouble())) {}
    }

    override fun didReceivePastActivity(aidlab: IAidlab, timestamp: Long, activity: ActivityType) {
        listener?.didReceivePastActivity(ActivityEvent(timestamp, activity.value.toLong())) {}
    }

    override fun didReceivePastBodyPosition(aidlab: IAidlab, timestamp: Long, bodyPosition: BodyPosition) {
        listener?.didReceivePastBodyPosition(BodyPositionEvent(timestamp, bodyPosition.value.toLong())) {}
    }

    override fun didReceivePastECG(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastECG(EcgEvent(timestamp, values.map { it.toDouble() }.toDoubleArray())) {}
    }

    override fun didReceivePastGyroscope(aidlab: IAidlab, timestamp: Long, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastGyroscope(VectorEvent(timestamp, qx.toDouble(), qy.toDouble(), qz.toDouble())) {}
    }

    override fun didReceivePastHeartRate(aidlab: IAidlab, timestamp: Long, heartRate: Int) {
        listener?.didReceivePastHeartRate(HeartRateEvent(timestamp, heartRate.toLong())) {}
    }

    override fun didReceivePastRr(aidlab: IAidlab, timestamp: Long, rr: Int) {
        listener?.didReceivePastRr(RrEvent(timestamp, rr.toLong())) {}
    }

    override fun didReceivePastMagnetometer(aidlab: IAidlab, timestamp: Long, mx: Float, my: Float, mz: Float) {
        listener?.didReceivePastMagnetometer(VectorEvent(timestamp, mx.toDouble(), my.toDouble(), mz.toDouble())) {}
    }

    override fun didReceivePastOrientation(aidlab: IAidlab, timestamp: Long, roll: Float, pitch: Float, yaw: Float) {
        listener?.didReceivePastOrientation(OrientationEvent(timestamp, roll.toDouble(), pitch.toDouble(), yaw.toDouble())) {}
    }

    override fun didReceivePastPressure(aidlab: IAidlab, timestamp: Long, values: IntArray) {
        listener?.didReceivePastPressure(PressureEvent(timestamp, values)) {}
    }

    override fun didReceivePastQuaternion(aidlab: IAidlab, timestamp: Long, qw: Float, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastQuaternion(QuaternionEvent(timestamp, qw.toDouble(), qx.toDouble(), qy.toDouble(), qz.toDouble())) {}
    }

    override fun didReceivePastRespiration(aidlab: IAidlab, timestamp: Long, values: FloatArray) {
        listener?.didReceivePastRespiration(RespirationEvent(timestamp, values.map { it.toDouble() }.toDoubleArray())) {}
    }

    override fun didReceivePastRespirationRate(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastRespirationRate(RespirationRateEvent(timestamp, value.toLong())) {}
    }

    override fun didReceivePastSignalQuality(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastSignalQuality(SignalQualityEvent(timestamp, value.toLong())) {}
    }

    override fun didReceivePastSkinTemperature(aidlab: IAidlab, timestamp: Long, value: Float) {
        listener?.didReceivePastSkinTemperature(SkinTemperatureEvent(timestamp, value.toDouble())) {}
    }

    override fun didReceivePastSoundFeatures(aidlab: IAidlab, values: FloatArray) {
        // listener?.didReceivePastSoundFeatures(SoundFeaturesEvent().apply {
        //     this.values = values.map { it.toDouble() }.toDoubleArray()
        // }) {}
        listener?.didReceivePastSoundFeatures(SoundFeaturesEvent(values.map { it.toDouble() }.toDoubleArray())) {}
    }

    override fun didReceivePastSoundVolume(aidlab: IAidlab, timestamp: Long, value: Int) {
        listener?.didReceivePastSoundVolume(SoundVolumeEvent(timestamp, value.toLong())) {}
    }

    override fun didReceivePastSteps(aidlab: IAidlab, timestamp: Long, value: Long) {
        listener?.didReceivePastSteps(StepsEvent(timestamp, value)) {}
    }

    override fun didReceiveUnsynchronizedSize(aidlab: IAidlab, unsynchronizedSize: Int, syncBytesPerSecond: Float) {
        listener?.didReceiveUnsynchronizedSize(UnsynchronizedSizeEvent(unsynchronizedSize.toLong(), syncBytesPerSecond.toDouble())) {}
    }

    override fun syncStateDidChange(aidlab: IAidlab, state: SyncState) {
        Log.d("AIDLAB_FLUTTER", "syncStateDidChange $state")
        listener?.syncStateDidChange(SyncStateEvent(state.value.toLong())) {}
    }
}