//
//  AidlabBluetoothManager.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//

import AidlabSDK
import CoreBluetooth

public class AidlabBluetoothManager: AidlabSDKDelegate {
    
    private var initialized: Bool = false
    private var aidlabSDK: AidlabSDK?
    private var detectedAidlabs: [CBPeripheral] = []
        
    var listener: PlatformAidlabSdkDelegate?
    
    func initial() {
        
        if initialized { return }
        initialized = true
        
        aidlabSDK = AidlabSDK(delegate: self)
    }
    
    func startScan(scanMode: ScanMode) {
        aidlabSDK?.scan(scanMode: scanMode)
        detectedAidlabs = []
    }
    
    func stopScan() {
        aidlabSDK?.stopScan()
    }
    
    func addressToAidlab(address: String) -> CBPeripheral? {
        return detectedAidlabs.first(where: { $0.identifier.uuidString == address })
    }
    
    func connect(signals: [Signal], aidlab: CBPeripheral, delegate: AidlabDelegate) {
        aidlabSDK?.connect(signals: signals, peripheral: aidlab, aidlabDelegate: delegate)
    }
    
    public func didDiscover(_ aidlab: CBPeripheral, rssi: NSNumber) {
        detectedAidlabs.append(aidlab)

        listener?.onAidlabDetected(event: AidlabDetectedEvent(address: aidlab.identifier.uuidString, rssi: Int64(truncating: rssi)), completion: { _ in })
    }
}
