//
//  AidlabSdkHandler.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//
import Flutter
import AidlabSDK

public class AidlabSdkHandler: PlatformAidlabSdk {
    private var aidlabSDKListener: PlatformAidlabSdkDelegate?
    private var aidlabHandler: AidlabHandler
    private var aidlabBluetoothManager: AidlabBluetoothManager

    init(aidlabHandler: AidlabHandler, aidlabBluetoothManager: AidlabBluetoothManager) {
        self.aidlabHandler = aidlabHandler
        self.aidlabBluetoothManager = aidlabBluetoothManager
    }

    func initSdk() throws {

    }

    func initial(binding: FlutterBinaryMessenger) {
        aidlabSDKListener = PlatformAidlabSdkDelegate(binaryMessenger: binding)
        PlatformAidlabSdkSetup.setUp(binaryMessenger: binding, api: self)
        aidlabBluetoothManager.listener = aidlabSDKListener
    }
    
    func release() {
        aidlabBluetoothManager.stopScan()
        aidlabBluetoothManager.listener = nil
    }
    
    func scanForDevices(scanMode: ScanModeRequest) throws {
        guard let scanMode = scanMode.mode else {
            print("Error: scanMode input = nil")
            return
        }
        aidlabBluetoothManager.startScan(scanMode: ScanMode(rawValue: Int(scanMode)) ?? .lowPower)
    }
    
    func stopDeviceScan() throws {
        aidlabBluetoothManager.stopScan()
    }
    
    func connect(request input: ConnectRequest) throws {

        aidlabBluetoothManager.stopScan()
        
        /// TODO throw exception
        guard let address = input.address else {
            print("Error: connect input = nil")
            return
        }
        
        guard let aidlab = aidlabBluetoothManager.addressToAidlab(address: address) else {
            print("Error: connect addressToAidlab")
            return
        }
        
        guard let rawSignals = input.signals?.data.toArray(type: Int32.self) else {
            print("Error: connect data to raw signals ")
            return
        }
        
        let signals = rawSignals.compactMap { Signal(rawValue: Int($0)) }
        aidlabBluetoothManager.connect(signals: signals, aidlab: aidlab, delegate: aidlabHandler)
    }
}
