//
//  AidlabSynchronizationHandler.swift
//  AidlabSDK
//
//  Created by Szymon Gęsicki on 05/02/2021.
//  Copyright © 2020 Aidlab. All rights reserved.
//

import Flutter
import UIKit
import AidlabSDK

public class AidlabSynchronizationHandler: AidlabSynchronizationDelegate {

    private var listener: PlatformAidlabSynchronizationDelegate?
    
    func initial(binding: FlutterBinaryMessenger) {
        listener = PlatformAidlabSynchronizationDelegate(binaryMessenger: binding)
    }
    
    func release() {
        listener = nil
    }
    
    public func syncStateDidChange(_ aidlab: IAidlab, state: SyncState) {
        listener?.syncStateDidChange(syncStateEvent: SyncStateEvent(state: Int64(state.rawValue)), completion: { _ in })
    }

    public func didReceivePastECG(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        let data = Data(values.withUnsafeBufferPointer { Data(buffer: $0) })
        listener?.didReceivePastECG(ecgEvent: EcgEvent(timestamp: Int64(timestamp), values: FlutterStandardTypedData(float32: data)), completion: { _ in
        })
    }

    public func didReceivePastRespiration(_ aidlab: IAidlab, timestamp: UInt64, values: [Float]) {
        let data = Data(values.withUnsafeBufferPointer { Data(buffer: $0) })
        let respirationEvent = RespirationEvent(timestamp: Int64(timestamp), values: FlutterStandardTypedData(float32: data))
        listener?.didReceivePastRespiration(respirationEvent: respirationEvent, completion: { _ in })
    }
    
    public func didReceivePastSkinTemperature(_ aidlab: IAidlab, timestamp: UInt64, value: Float) {

        let skinTemperatureEvent = SkinTemperatureEvent(timestamp: Int64(timestamp), value: Double(value))
        listener?.didReceivePastSkinTemperature(skinTemperatureEvent: skinTemperatureEvent, completion: { _ in })
    }
    
    public func didReceivePastHeartRate(_ aidlab: IAidlab, timestamp: UInt64, heartRate: Int32) {

        let heartRateEvent = HeartRateEvent(timestamp: Int64(timestamp), heartRate: Int64(heartRate))
        listener?.didReceivePastHeartRate(heartRateEvent: heartRateEvent, completion: { _ in })
    }
    
    public func didReceivePastRr(_ aidlab: IAidlab, timestamp: UInt64, rr: Int32) {

        let rrEvent = RrEvent(timestamp: Int64(timestamp), rr: Int64(rr))
        listener?.didReceivePastRr(rrEvent: rrEvent, completion: { _ in })
    }

    public func didReceiveUnsynchronizedSize(_ aidlab: IAidlab, unsynchronizedSize: UInt32, syncBytesPerSecond: Float) {
        listener?.didReceiveUnsynchronizedSize(unsynchronizedSizeEvent: UnsynchronizedSizeEvent(unsynchronizedSize: Int64(unsynchronizedSize), syncBytesPerSecond: Double(syncBytesPerSecond)), completion: { _ in })
    }
    
    public func didReceivePastRespirationRate(_ aidlab: IAidlab, timestamp: UInt64, value: UInt32) {
        listener?.didReceivePastRespirationRate(respirationRateEvent: RespirationRateEvent(timestamp: Int64(timestamp), value: Int64(value)), completion: { _ in })
    }
    
    public func didReceivePastActivity(_ aidlab: IAidlab, timestamp: UInt64, activity: ActivityType) {
        listener?.didReceivePastActivity(activityEvent: ActivityEvent(timestamp: Int64(timestamp), activityType: Int64(activity.rawValue)), completion: { _ in })
    }
    
    public func didReceivePastSteps(_ aidlab: IAidlab, timestamp: UInt64, value: UInt64) {
        listener?.didReceivePastSteps(stepsEvent: StepsEvent(timestamp: Int64(timestamp), value: Int64(value)), completion: { _ in })
    }
    
    public func didReceivePastSoundVolume(_ aidlab: IAidlab, timestamp: UInt64, soundVolume: UInt16) {
        listener?.didReceivePastSoundVolume(soundVolumeEvent: SoundVolumeEvent(timestamp: Int64(timestamp), soundVolume: Int64(soundVolume)), completion: { _ in })
    }
    
    public func didReceivePastAccelerometer(_ aidlab: IAidlab, timestamp: UInt64, ax: Float, ay: Float, az: Float) {
        listener?.didReceivePastAccelerometer(vectorEvent: VectorEvent(timestamp: Int64(timestamp), x: Double(ax), y: Double(ay), z: Double(az)), completion: { _ in })
    }
    
    public func didReceivePastGyroscope(_ aidlab: IAidlab, timestamp: UInt64, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastGyroscope(vectorEvent: VectorEvent(timestamp: Int64(timestamp), x: Double(qx), y: Double(qy), z: Double(qz)), completion: { _ in })
    }
    
    public func didReceivePastMagnetometer(_ aidlab: IAidlab, timestamp: UInt64, mx: Float, my: Float, mz: Float) {
        listener?.didReceivePastMagnetometer(vectorEvent: VectorEvent(timestamp: Int64(timestamp), x: Double(mx), y: Double(my), z: Double(mz)), completion: { _ in })
    }
    
    public func didReceivePastQuaternion(_ aidlab: IAidlab, timestamp: UInt64, qw: Float, qx: Float, qy: Float, qz: Float) {
        listener?.didReceivePastQuaternion(quaternionEvent: QuaternionEvent(timestamp: Int64(timestamp), qw: Double(qw), qx: Double(qx), qy: Double(qy), qz: Double(qz)), completion: { _ in })
    }
    
    public func didReceivePastOrientation(_ aidlab: IAidlab, timestamp: UInt64, roll: Float, pitch: Float, yaw: Float) {
        listener?.didReceivePastOrientation(orientationEvent: OrientationEvent(timestamp: Int64(timestamp), roll: Double(roll), pitch: Double(pitch), yaw: Double(yaw)), completion: { _ in })
    }
    
    public func didReceivePastBodyPosition(_ aidlab: IAidlab, timestamp: UInt64, bodyPosition: BodyPosition) {
        listener?.didReceivePastBodyPosition(bodyPositionEvent: BodyPositionEvent(timestamp: Int64(timestamp), bodyPosition: Int64(bodyPosition.rawValue)), completion: { _ in })
    }
    
    public func didReceivePastPressure(_ aidlab: IAidlab, timestamp: UInt64, values: [Int32]) {
        let data = Data(values.withUnsafeBufferPointer { Data(buffer: $0) })
        listener?.didReceivePastPressure(pressureEvent: PressureEvent(timestamp: Int64(timestamp), values: FlutterStandardTypedData(float32: data)), completion: { _ in })
    }
    
    public func didReceivePastSoundFeatures(_ aidlab: IAidlab, values: [Float]) {
        let data = Data(values.withUnsafeBufferPointer { Data(buffer: $0) })
        listener?.didReceivePastSoundFeatures(soundFeaturesEvent: SoundFeaturesEvent(values: FlutterStandardTypedData(float32: data)), completion: { _ in })
    }
    
    public func didDetectPastUserEvent(_ aidlab: IAidlab, timestamp: UInt64) {
        listener?.didDetectPastUserEvent(userEvent: UserEvent(timestamp: Int64(timestamp)), completion: { _ in })
    }
    
    public func didReceivePastSignalQuality(_ timestamp: UInt64, value: Int32) {
        listener?.didReceivePastSignalQuality(signalQualityEvent: SignalQualityEvent(timestamp: Int64(timestamp), value: Int64(value)), completion: { _ in })
    }
}
