import 'package:pigeon/pigeon.dart';

@ConfigurePigeon(PigeonOptions(
  dartOut: 'lib/src/wrappers/platform_aidlab_sdk.dart',
  kotlinOut:
      'android/src/main/kotlin/com/aidlab/aidlab_sdk/PlatformAidlabSdk.kt',
  kotlinOptions: KotlinOptions(package: 'com.aidlab.aidlab_sdk'),
  swiftOut: 'ios/Classes/PlatformAidlabSdk.swift',
))
@HostApi()
abstract class PlatformAidlabSdk {
  void initSdk();

  void scanForDevices(ScanModeRequest scanMode);

  void stopDeviceScan();

  void connect(ConnectRequest request);
}

@FlutterApi()
abstract class PlatformAidlabSdkDelegate {
  void onBluetoothStarted(EmptyEvent voidEvent);

  void onDeviceScanStarted(EmptyEvent voidEvent);

  void onDeviceScanStopped(EmptyEvent voidEvent);

  void onScanFailed(ScanFailedEvent event);

  void onAidlabDetected(AidlabDetectedEvent event);
}

class ScanModeRequest {
  int? mode;
}

class ConnectRequest {
  String? address;
  Int32List? signals;
}

class AidlabDetectedEvent {
  String? address;
  int? rssi;
}

class ScanFailedEvent {
  int? errorCode;
}

@FlutterApi()
abstract class PlatformAidlabLogger {
  void onLog(LogEvent event);

  void onWarning(LogEvent event);

  void onError(LogEvent event);
}

class LogEvent {
  String? msg;
}

@FlutterApi()
abstract class PlatformAidlabDelegate {
  void didConnect(ConnectEvent connectEvent);

  void didDisconnect(DisconnectEvent voidEvent);

  void didReceiveEcg(EcgEvent ecgEvent);

  void didReceiveRespiration(RespirationEvent respirationEvent);

  void didReceiveRespirationRate(RespirationRateEvent respirationRateEvent);

  void didReceiveBatteryLevel(BatteryLevelEvent batteryLevelEvent);

  void didReceiveSkinTemperature(SkinTemperatureEvent skinTemperatureEvent);

  void didReceiveAccelerometer(VectorEvent vectorEvent);

  void didReceiveGyroscope(VectorEvent vectorEvent);

  void didReceiveMagnetometer(VectorEvent vectorEvent);

  void didReceiveQuaternion(QuaternionEvent quaternionEvent);

  void didReceiveOrientation(OrientationEvent orientationEvent);

  void didReceiveBodyPosition(BodyPositionEvent bodyPositionEvent);

  void didReceiveActivity(ActivityEvent activityEvent);

  void didReceiveSteps(StepsEvent stepsEvent);

  void didReceiveHeartRate(HeartRateEvent heartRateEvent);

  void didReceiveRr(RrEvent rrEvent);

  void wearStateDidChange(WearStateEvent wearStateEvent);

  void didDetectExercise(ExerciseEvent exerciseEvent);

  void didReceiveSoundVolume(SoundVolumeEvent soundVolumeEvent);

  void didReceiveCommand(EmptyEvent voidEvent);

  void didReceiveError(ErrorEvent errorEvent);

  void didReceiveMessage(MessageEvent messageEvent);

  void didDetectUserEvent(UserEvent userEvent);

  void didReceiveSoundFeatures(SoundFeaturesEvent soundFeaturesEvent);

  void didReceiveSignalQuality(SignalQualityEvent signalQualityEvent);
}

class ConnectEvent {
  late String firmwareRevision;
  late String hardwareRevision;
  late String serialNumber;
  late String address;
}

class EcgEvent {
  late int timestamp;
  late Float64List values;
}

class RespirationEvent {
  late int timestamp;
  late Float64List values;
}

class RespirationRateEvent {
  late int timestamp;
  late int value;
}

class BatteryLevelEvent {
  late int stateOfCharge;
}

class VectorEvent {
  late int timestamp;
  late double x;
  late double y;
  late double z;
}

class QuaternionEvent {
  late int timestamp;
  late double qw;
  late double qx;
  late double qy;
  late double qz;
}

class OrientationEvent {
  late int timestamp;
  late double roll;
  late double pitch;
  late double yaw;
}

class BodyPositionEvent {
  late int timestamp;
  late int bodyPosition;
}

class ActivityEvent {
  late int timestamp;
  late int activityType;
}

class StepsEvent {
  late int timestamp;
  late int value;
}

class HeartRateEvent {
  late int timestamp;
  late int heartRate;
}

class RrEvent {
  late int timestamp;
  late int rr;
}

class WearStateEvent {
  late int wearState;
}

class SkinTemperatureEvent {
  late int timestamp;
  late double value;
}

class ExerciseEvent {
  int? exercise;
}

class SoundVolumeEvent {
  late int timestamp;
  late int soundVolume;
}

class ErrorEvent {
  late String error;
}

class MessageEvent {
  late String process;
  late String message;
}

class UserEvent {
  late int timestamp;
}

class SoundFeaturesEvent {
  late Float64List values;
}

class DisconnectEvent {
  late int disconnectReason;
}

class SignalQualityEvent {
  late int timestamp;
  late int value;
}

@FlutterApi()
abstract class PlatformAidlabInternalDelegate {
  void didReceiveFirmwareInformation(
      FirmwareInformationEvent firmwareInformationEvent);

  void didReceivePressure(PressureEvent pressureEvent);

  void pressureWearStateDidChange(PressureWearStateEvent wearStateEvent);
}

@FlutterApi()
abstract class PlatformAidlabUpdateDelegate {
  void didFinishUpdate(EmptyEvent nothing);

  void didReceiveUpdateError(UpdateEvent event);

  void didReceiveUpdateProgress(UpdateProgress progress);
}

class FirmwareInformationEvent {
  bool? updateAvailable;
  String? updatableVersion;
}

class PressureEvent {
  int? timestamp;
  Int32List? values;
}

class PressureWearStateEvent {
  int? wearState;
}

class UpdateProgress {
  int? progress;
}

class UpdateEvent {
  int? error;
}

@HostApi()
abstract class PlatformAidlab {
  void disconnect();

  void send(SendRequest sendRequest);

  void setECGFiltrationMethod(EcgFiltrationRequest ecgFiltrationRequest);

  void startSynchronization();

  void stopSynchronization();

  void requestForFirmwareInfo(FirmwareInfoRequest firmwareInfoRequest);

  void startFirmwareUpdate(FirmwareUpdateRequest firmwareUpdateRequest);
}

class SendRequest {
  late String command;
}

class EcgFiltrationRequest {
  late int method;
}

class FirmwareInfoRequest {
  late String url;
}

class FirmwareUpdateRequest {
  late String url;
  late String updatableVersion;
}

@FlutterApi()
abstract class PlatformAidlabSynchronizationDelegate {
  void syncStateDidChange(SyncStateEvent syncStateEvent);

  void didReceivePastECG(EcgEvent ecgEvent);

  void didReceivePastRespiration(RespirationEvent respirationEvent);

  void didReceivePastSkinTemperature(SkinTemperatureEvent skinTemperatureEvent);

  void didReceivePastHeartRate(HeartRateEvent heartRateEvent);

  void didReceivePastRr(RrEvent rrEvent);

  void didReceiveUnsynchronizedSize(
      UnsynchronizedSizeEvent unsynchronizedSizeEvent);

  void didReceivePastRespirationRate(RespirationRateEvent respirationRateEvent);

  void didReceivePastActivity(ActivityEvent activityEvent);

  void didReceivePastSteps(StepsEvent stepsEvent);

  void didReceivePastSoundVolume(SoundVolumeEvent soundVolumeEvent);

  void didReceivePastAccelerometer(VectorEvent vectorEvent);

  void didReceivePastGyroscope(VectorEvent vectorEvent);

  void didReceivePastMagnetometer(VectorEvent vectorEvent);

  void didReceivePastQuaternion(QuaternionEvent quaternionEvent);

  void didReceivePastOrientation(OrientationEvent orientationEvent);

  void didReceivePastBodyPosition(BodyPositionEvent bodyPositionEvent);

  void didReceivePastPressure(PressureEvent pressureEvent);

  void didDetectPastUserEvent(UserEvent userEvent);

  void didReceivePastSoundFeatures(SoundFeaturesEvent soundFeaturesEvent);

  void didReceivePastSignalQuality(SignalQualityEvent signalQualityEvent);
}

class SyncStateEvent {
  late int state;
}

class UnsynchronizedSizeEvent {
  late int unsynchronizedSize;
  late double syncBytesPerSecond;
}

class EmptyEvent {
  late int nothing;
}
